from typing import List, Optional
import sublime_plugin
import sublime

string_scope = 'meta.tag.attributes string.quoted'

class InlineFoldListener(sublime_plugin.ViewEventListener):
	def __init__(self, view):
		super().__init__(view)
		self._stored_region = sublime.Region(-1, -1)
		self.prev_string_region = None

	def on_load(self) -> None:
		self.view.run_command('inline_fold_all')

	def on_selection_modified(self) -> None:
		r = first_selection_region(self.view)
		if r is None:
			return
		if self.prev_string_region is not None and not self.view.match_selector(r.b, string_scope):
			start = self.view.find(r"""("|')""", self.prev_string_region.begin())
			end = self.view.find(r"""("|')""", self.prev_string_region.end() - 1)
			fold_r = sublime.Region(start.begin() + 1, end.end() - 1)
			self.view.fold(fold_r)
		string_region = self.view.expand_to_scope(r.b, string_scope)
		self.prev_string_region = string_region
		if string_region is None:
			return
		self.view.unfold(string_region)
		self.prev_string_region = string_region


class InlineFoldAll(sublime_plugin.TextCommand):
	def run(self, _: sublime.Edit) -> None:
		regions = find_regions(self.view)
		for r in regions:
			self.view.fold(r)


class InlineUnfoldAll(sublime_plugin.TextCommand):
	def run(self, _: sublime.Edit) -> None:
		regions = find_regions(self.view)
		for r in regions:
			self.view.unfold(r)

def first_selection_region(view: sublime.View) -> Optional[sublime.Region]:
    try:
        return view.sel()[0]
    except IndexError:
        return None

def find_regions(view: sublime.View) -> List[sublime.Region]:
	return list(map(lambda r: sublime.Region(r.begin()+1, r.end() - 1), view.find_by_selector(string_scope)))